/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.service.impl;

import java.util.Collection;

import org.appspy.server.bo.Application;
import org.appspy.server.bo.ApplicationVersion;
import org.appspy.server.bo.Environment;
import org.appspy.server.bo.Host;
import org.appspy.server.bo.Organization;
import org.appspy.server.dao.ApplicationDao;
import org.appspy.server.dao.EnvironmentDao;
import org.appspy.viewer.service.SettingsService;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class SettingsServiceImpl implements SettingsService {
	
	protected EnvironmentDao mEnvironmentDao = null;
	protected ApplicationDao mApplicationDao = null;
	protected PlatformTransactionManager mTransactionManager = null;

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#createEnvironment(org.appspy.admin.form.EnvironmentForm)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Environment createEnvironment(Environment env) {

		return mEnvironmentDao.saveEnvironment(env);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#updateEnvironment(org.appspy.admin.form.EnvironmentForm)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Environment updateEnvironment(Environment env) {
		
		Environment current = mEnvironmentDao.findEnvironmentById(env.getId());
		env.getHosts().addAll(current.getHosts());
		
		return mEnvironmentDao.updateEnvironment(env);
	}
	
	

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#deleteEnvironment(org.appspy.server.bo.Environment)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public void deleteEnvironment(Environment env) {
		mEnvironmentDao.deleteEnvironment(env);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#getEnvironments()
	 */
	public Collection<Environment> getEnvironments() {
		return mEnvironmentDao.findAllEnvironments(true);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#createOrganization(org.appspy.server.bo.Organization)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Organization createOrganization(Organization org) {
		
		return mEnvironmentDao.saveOrganization(org);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#updateOrganization(org.appspy.server.bo.Organization)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Organization updateOrganization(Organization org) {
		return mEnvironmentDao.updateOrganization(org);
	}
	
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#deleteOrganization(org.appspy.server.bo.Organization)
	 */
	public void deleteOrganization(Organization org) {
		mEnvironmentDao.deleteOrganization(org);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#getOrganizations()
	 */
	public Collection<Organization> getOrganizations() {
		return mEnvironmentDao.findAllOrganizations();
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#createHost(org.appspy.server.bo.Host)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Host createHost(Host host) {
		return mEnvironmentDao.saveHost(host);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#deleteHost(org.appspy.server.bo.Host)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public void deleteHost(Host host) {
		mEnvironmentDao.deleteHost(host);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#getHosts()
	 */
	public Collection<Host> getHosts() {
		return mEnvironmentDao.findAllHosts(true);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#updateHost(org.appspy.server.bo.Host)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Host updateHost(Host host) {
		return mEnvironmentDao.updateHost(host);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#createApplication(org.appspy.server.bo.Application)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Application createApplication(Application app) {
		return mApplicationDao.saveApplication(app);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#deleteApplication(org.appspy.server.bo.Application)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public void deleteApplication(Application app) {
		mApplicationDao.removeApplication(app);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#getApplications()
	 */
	public Collection<Application> getApplications() {
		return mApplicationDao.findAllApplications();
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#updateApplication(org.appspy.server.bo.Application)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public Application updateApplication(Application app) {
		return mApplicationDao.updateApplication(app);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#createVersion(org.appspy.server.bo.ApplicationVersion)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public ApplicationVersion createVersion(ApplicationVersion ver) {
		return mApplicationDao.saveVersion(ver);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#deleteVersion(org.appspy.server.bo.ApplicationVersion)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public void deleteVersion(ApplicationVersion ver) {
		mApplicationDao.removeVersion(ver);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#getVersions()
	 */
	public Collection<ApplicationVersion> getVersions() {
		return mApplicationDao.findAllVersions();
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.service.SettingsService#updateVersion(org.appspy.server.bo.ApplicationVersion)
	 */
	@Transactional(readOnly=false, propagation=Propagation.REQUIRED)
	public ApplicationVersion updateVersion(ApplicationVersion ver) {
		return mApplicationDao.updateVersion(ver);
	}

	/**
	 * @return the environmentDao
	 */
	public EnvironmentDao getEnvironmentDao() {
		return mEnvironmentDao;
	}

	/**
	 * @param environmentDao the environmentDao to set
	 */
	public void setEnvironmentDao(EnvironmentDao environmentDao) {
		mEnvironmentDao = environmentDao;
	}
	
	/**
	 * @return the applicationDao
	 */
	public ApplicationDao getApplicationDao() {
		return mApplicationDao;
	}

	/**
	 * @param applicationDao the applicationDao to set
	 */
	public void setApplicationDao(ApplicationDao applicationDao) {
		mApplicationDao = applicationDao;
	}

	/**
	 * @return the transactionManager
	 */
	public PlatformTransactionManager getTransactionManager() {
		return mTransactionManager;
	}

	/**
	 * @param transactionManager the transactionManager to set
	 */
	public void setTransactionManager(PlatformTransactionManager transactionManager) {
		mTransactionManager = transactionManager;
	}
	
	

}
